<!-- "Add Cell" dialog -->
<div class="modal-header">
  <h4 class="modal-title">Select New Cell Type</h4>
</div>
<div class="modal-body">
  <div class="list-group">
    <a ng-repeat="(typeKey, type) in items" href="#" data-dismiss="modal" class="list-group-item" ng-click="ok(typeKey)">{{type.description}}</a>
  </div>
</div>
