cmd_/home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.o := gcc -Wp,-MD,/home/scudette/rekall/tools/linux/lmap/build/modules/.pte_mmap.o.d  -nostdinc -isystem /usr/lib/gcc/x86_64-linux-gnu/4.6/include  -I/home/scudette/rekall/tools/linux/lmap/build/kernel_headers/arch/x86/include -Iarch/x86/include/generated  -Iinclude -I/home/scudette/rekall/tools/linux/lmap/build/kernel_headers/arch/x86/include/uapi -Iarch/x86/include/generated/uapi -I/home/scudette/rekall/tools/linux/lmap/build/kernel_headers/include/uapi -Iinclude/generated/uapi -include /home/scudette/rekall/tools/linux/lmap/build/kernel_headers/include/linux/kconfig.h -Iubuntu/include  -D__KERNEL__ -Wall -Wundef -Wstrict-prototypes -Wno-trigraphs -fno-strict-aliasing -fno-common -Werror-implicit-function-declaration -Wno-format-security -fno-delete-null-pointer-checks -O2 -m64 -mno-mmx -mno-sse -mtune=generic -mno-red-zone -mcmodel=kernel -funit-at-a-time -maccumulate-outgoing-args -fstack-protector -DCONFIG_X86_X32_ABI -DCONFIG_AS_CFI=1 -DCONFIG_AS_CFI_SIGNAL_FRAME=1 -DCONFIG_AS_CFI_SECTIONS=1 -DCONFIG_AS_FXSAVEQ=1 -DCONFIG_AS_AVX=1 -DCONFIG_AS_AVX2=1 -pipe -Wno-sign-compare -fno-asynchronous-unwind-tables -mno-sse -mno-mmx -mno-sse2 -mno-3dnow -mno-avx -Wframe-larger-than=1024 -Wno-unused-but-set-variable -fno-omit-frame-pointer -fno-optimize-sibling-calls -pg -mfentry -DCC_USING_FENTRY -Wdeclaration-after-statement -Wno-pointer-sign -fno-strict-overflow -fconserve-stack -Werror=implicit-int -Werror=strict-prototypes -DCC_HAVE_ASM_GOTO -std=gnu99 -O2  -DMODULE  -D"KBUILD_STR(s)=\#s" -D"KBUILD_BASENAME=KBUILD_STR(pte_mmap)"  -D"KBUILD_MODNAME=KBUILD_STR(pmem_pte)" -c -o /home/scudette/rekall/tools/linux/lmap/build/modules/.tmp_pte_mmap.o /home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.c

source_/home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.o := /home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.c

deps_/home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.o := \
  /home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.h \
  /usr/lib/gcc/x86_64-linux-gnu/4.6/include/stddef.h \

/home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.o: $(deps_/home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.o)

$(deps_/home/scudette/rekall/tools/linux/lmap/build/modules/pte_mmap.o):
